#include <boost/thread.hpp>
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <ctime>
#include "MyCounterP.h"

/*
 *	This program creates one pthread that increases a MyCounterP object
 *	until a given goal; once with a mutex lock, once without
 */

struct threadData
{
  MyCounter *mc;
  int mode;
};

void *thread_method(void *ptr)
{
  struct threadData *td;

  td = (struct threadData *) ptr;
  MyCounter *mc = td->mc;
  int mode = td->mode;
  
  bool result = false;
  
  while(!result)
  {
    if(mode == 0)
    {
      result = mc->Increase();
    }else
    {
      result = mc->safeIncrease();
    }
  }
}

long launchThreads(int threadNumber, int mode, MyCounter *mc)
{
   struct timeval start, end;

   long mtime, seconds, useconds;  
   
   pthread_t pt[threadNumber];
   
   pthread_t x;
   // Assemble thread arguments
   struct threadData td;
   td.mode = mode;
   td.mc = mc;
   
  // Start the timer
  gettimeofday(&start, NULL);
  
  for(int i = 0; i < threadNumber; i++)
  {
    pthread_create(&pt[i], NULL, thread_method, (void*) &td);
  }
  for(int i = 0; i < threadNumber; i++)
  {
    pthread_join(pt[i], NULL);
  }
  // Stop the timer
  gettimeofday(&end, NULL);
  
  seconds  = end.tv_sec  - start.tv_sec;
  useconds = end.tv_usec - start.tv_usec;
  mtime = ((seconds) * 1000 + useconds/1000.0) + 0.5; 
  
  return(mtime);
}

main(int argc, char* argv[])
{

  int goal = atoi(argv[1]);
  
  int threadNumber = 1;
  
  long elapsedTime = 0;
 
  
  MyCounter *mc = new MyCounter(goal);

  elapsedTime = launchThreads(threadNumber, 0, mc);
  
  printf("Elapsed time (unlocked): %ld ms\n", elapsedTime);
 
  mc->value = 0;
  
  elapsedTime = launchThreads(threadNumber, 1, mc);
  
  printf("Elapsed time (locked): %ld ms\n", elapsedTime);

}


