#include <boost/thread.hpp>
#include <stdlib.h>
#include <stdio.h>
#include <ctime>
#include "MyCounter.h"

/*
 *	This program runs the Locking Test on boost threads several times for an increasing goal
 */
  
void threadFunction(MyCounter *mc, int mode)
{
  bool result = false;

  while(!result)
  {
    if(mode == 0){
      result = mc->Increase();
    }else
    {
      result = mc->safeIncrease();
    }
  }
}


long launchThreads(int threadNumber, int mode, MyCounter *mc)
{
   struct timeval start, end;

   long mtime, seconds, useconds;  
   
   boost::thread* threads[threadNumber];
   
  // Start the timer
  gettimeofday(&start, NULL);
  
  for(int i = 0; i < threadNumber; i++)
  {
    threads[i] = new boost::thread(threadFunction, mc, mode);
  }
  for(int i = 0; i < threadNumber; i++)
  {
    threads[i]->join();
  }
  // Stop the timer
  gettimeofday(&end, NULL);
  
  seconds  = end.tv_sec  - start.tv_sec;
  useconds = end.tv_usec - start.tv_usec;
  mtime = ((seconds) * 1000 + useconds/1000.0) + 0.5; 
  
  return(mtime);
}


int main(int argc, char* argv[])
{
  int steps[] = {10000, 50000, 100000, 500000, 1000000, 10000000, 100000000, 500000000};
  
  int stepsNumber = 8;
  
  int threadNumber = 1;
  
  long elapsedTime;
 
  
  MyCounter *mc = new MyCounter(0);

  for(int i = 0; i < stepsNumber; i++)
  {
    mc->goal = steps[i];
    
    elapsedTime = launchThreads(threadNumber, 0, mc);
  
    printf("Goal %d\tElapsed time (unlocked): %ld ms\n",steps[i], elapsedTime);
 
    mc->value = 0;
  
    elapsedTime = launchThreads(threadNumber, 1, mc);
  
    printf("\t\tElapsed time (locked): \t%ld ms\n\n", elapsedTime);
    
    mc->value = 0;
   }
}

