#include <boost/thread.hpp>
#include <stdlib.h>
#include <stdio.h>
#include <ctime>
#include "MyCounterP.h"

/*
 *	This program runs the Locking Test on pthreads several times for an increasing goal
 */

struct threadData
{
  MyCounter *mc;
  int mode;
};

void *thread_method(void *ptr)
{
  struct threadData *td;

  td = (struct threadData *) ptr;
  MyCounter *mc = td->mc;
  int mode = td->mode;
  
  bool result = false;
  
  while(!result)
  {
    if(mode == 0)
    {
      result = mc->Increase();
    }else
    {
      result = mc->safeIncrease();
    }
  }
}


long launchThreads(int threadNumber, int mode, MyCounter *mc)
{
   struct timeval start, end;

   long mtime, seconds, useconds;  
   
   pthread_t pt[threadNumber];
   
   pthread_t x;
   // Assemble thread arguments
   struct threadData td;
   td.mode = mode;
   td.mc = mc;
   
  // Start the timer
  gettimeofday(&start, NULL);
  
  for(int i = 0; i < threadNumber; i++)
  {
    pthread_create(&pt[i], NULL, thread_method, (void*) &td);
  }
  for(int i = 0; i < threadNumber; i++)
  {
    pthread_join(pt[i], NULL);
  }
  // Stop the timer
  gettimeofday(&end, NULL);
  
  seconds  = end.tv_sec  - start.tv_sec;
  useconds = end.tv_usec - start.tv_usec;
  mtime = ((seconds) * 1000 + useconds/1000.0) + 0.5; 
  
  return(mtime);
}


int main(int argc, char* argv[])
{
  int steps[] = {10000, 50000, 100000, 500000, 1000000, 10000000, 100000000, 500000000};
  
  int stepsNumber = 8;
  
  int threadNumber = 1;
  
  long elapsedTime;
 
  
  MyCounter *mc = new MyCounter(0);

  for(int i = 0; i < stepsNumber; i++)
  {
    mc->goal = steps[i];
    
    elapsedTime = launchThreads(threadNumber, 0, mc);
  
    printf("Goal %d\tElapsed time (unlocked): %ld ms\n",steps[i], elapsedTime);
 
    mc->value = 0;
  
    elapsedTime = launchThreads(threadNumber, 1, mc);
  
    printf("\t\tElapsed time (locked): \t%ld ms\n\n", elapsedTime);
    
    mc->value = 0;
   }
}

